import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;

/**
 * Asteroid
 * Just provides easier usage for collision and spawning of Asteroids
 * 
 * @author Jannis
 * @version 1.0
 */
public class Asteroid extends HelperClass
{
    // Minimum ditance from the corners of the World
    private int flex = 100;
    
    int asteroidType = 1;
    
    //idk why but this needs to wait a bit before it can die so the shot can register the touch too
    private int time = 1; 
    
    // Make the speed of the random from 0.3 - 0.7
    public double movementSpeed = (Greenfoot.getRandomNumber(40)+30)/(double)100;
    
    private int health;
    
    int p;
    
    /**
     * Constructor for asteroid, set type here
     * 1: small, 2: medium, 3: big, default: 1
     */
    public Asteroid(int type) {
        asteroidType = type;
        switch (type) {
            case 1:
                setImage("asteroid(small).png");
                break;
                
            case 2:
                setImage("asteroid(medium).png");
                break;
                
            case 3:
                setImage("asteroid(big).png");
                break;
                
            default:
                asteroidType = 1;
                setImage("asteroid(small).png");
                break;
        }
        health = getHealth();
    }
    /**
     * Get the type of asteroid
     */
    public int getType() {
        return asteroidType;
    }
    /**
     * Act - do whatever the BigAsteroid wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        //Ceck for game over
        if ( ! getWorld().getObjects(Button.class).isEmpty() && !SettingsScreen.class.isInstance(getWorld())) return;
        
        move(movementSpeed);
        // Removes itself if at edge
        if (isAtEdge()){
            getWorld().removeObject(this);
            return;
        }
        if (time > 1) {
            time --;
        }
        
        ArrayList<Integer> n = touchAnyShot(time, health, p);
        time = n.get(0);
        health = n.get(1);
    }
    /**
     * This method is called by the Greenfoot system when this actor has been inserted into the world.
     */
    protected void addedToWorld​(World world) {
        if (Settings.AsteroidsGrantPoints) {p = getPoints();} else {p = 0;}
        
        // Random side of the map the Asteroid will spawn on
        int side = Greenfoot.getRandomNumber(4);
        if (side == 0) {
            // top
            setLocation(Greenfoot.getRandomNumber(getWorld().getWidth()-flex)+flex, 1);
            setRotation(Greenfoot.getRandomNumber(90)+45);
        } else if (side == 1) {
            // right
            setLocation(getWorld().getWidth()-1, Greenfoot.getRandomNumber(getWorld().getHeight()-flex)+flex);
            setRotation(Greenfoot.getRandomNumber(90)+135);
        } else if (side == 2) {
            // bottom
            setLocation(Greenfoot.getRandomNumber(getWorld().getWidth()-flex)+flex, getWorld().getHeight()-1);
            setRotation(Greenfoot.getRandomNumber(90)+225);
        } else {
            // left
            setLocation(1, Greenfoot.getRandomNumber(getWorld().getHeight()-flex)+flex);
            setRotation(Greenfoot.getRandomNumber(90)-45);
        }
    }
    /**
     * Get the Health of the current asteroid type
     */
    public int getHealth(){
        switch (asteroidType) {
            case 1:
                return Settings.SmallAsteroidHealth;
            case 2:
                return Settings.MediumAsteroidHealth;
            case 3:
                return Settings.BigAsteroidHealth;
            default:
                return Settings.SmallAsteroidHealth;
        }
    }
    /**
     * Get the Points for destroying the current asteroid type
     */
    public int getPoints(){
        switch (asteroidType) {
            case 1:
                return Settings.SmallAsteroidPoints;
            case 2:
                return Settings.MediumAsteroidPoints;
            case 3:
                return Settings.BigAsteroidPoints;
            default:
                return Settings.SmallAsteroidPoints;
        }
    }
}
