import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Enemies
 * Just provides easier usage for collision and some shared methods
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class Enemies extends HelperClass {
    /**
     * Update healthbar if the enemy has more than 1 maxHealth
     * bar: health bar
     * MaxHealth: maximum health
     * health: current health
     */
    public Bar updateHealthBar(Bar bar, int MaxHealth, int health) {
        if (MaxHealth > 1 && Settings.EnemiesShowHealthBar) {
            if (bar == null) {  
                bar = new Bar("", "", 100, 100);
                getWorld().addObject(bar, 0, 0);
                bar.setBreakValue(25);
            }
            bar.setLocation(getX(), getY());
            bar.moveSide(50);
            bar.setRotation(0);
            double v = health/(double)MaxHealth*100;
            bar.setValue((int)v);
            
            return bar;
        }
        return null;
    }
    /**
     * Show and move exhaust if turned on
     * exhaust: current exhaust
     */
    public Exhaust moveExhaust(Exhaust exhaust) {
        if(Settings.EnemiesHaveExhausts){
            exhaust.setLocation(getX(), getY());
            exhaust.move(-40);
            exhaust.setRotation(getRotation());
        }
        return exhaust;
    }
    /**
     * Launch a shot towards the spaceship
     * rotation: original rotation
     * x: x-location
     * y: y-location
     */
    public void launchShot(int rotation, int x, int y) {

        Shots actor = new Shots(0, false, 2);
        actor.setColor("Green");
            
            
        getWorld().addObject(actor, x, y);
            
        //actor.move(145);
        //actor.moveSide(-20);
        actor.setRotation(rotation);
          
        Spaceship sp = getWorld().getObjects(Spaceship.class).get(0);
        turnTowards(sp.getX(), sp.getY());
    }
    /**
     * This method is called by the Greenfoot system when this actor has been inserted into the world.
     */
    protected void addedToWorld​(World world) {
        // Minimum distance from the corners of the World
        int flex = 100;
        
        // Random side of the map the Enemy will spawn on
        int side = Greenfoot.getRandomNumber(4);
        if (side == 0) {
            // top
            setLocation(Greenfoot.getRandomNumber(getWorld().getWidth()-flex)+flex, 10);
            setRotation(Greenfoot.getRandomNumber(90)+45);
        } else if (side == 1) {
            // right
            setLocation(getWorld().getWidth()-10, Greenfoot.getRandomNumber(getWorld().getHeight()-flex)+flex);
            setRotation(Greenfoot.getRandomNumber(90)+135);
        } else if (side == 2) {
            // bottom
            setLocation(Greenfoot.getRandomNumber(getWorld().getWidth()-flex)+flex, getWorld().getHeight()-10);
            setRotation(Greenfoot.getRandomNumber(90)+225);
        } else {
            // left
            setLocation(10, Greenfoot.getRandomNumber(getWorld().getHeight()-flex)+flex);
            setRotation(Greenfoot.getRandomNumber(90)-45);
        } 
        startUpExtras(world);
    }
    /**
     * This method is called by the Enemies addedToWorld, on object spawn
     * Override this method in a subclass for addistion spawn option
     */
    public void startUpExtras​(World world) {}
}
