import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Exhaust
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class Exhaust extends HelperClass
{
    private boolean boost = false;
    private boolean engine = false;
    private boolean engineRampedUp = false;
    private String color = Settings.SpaceshipExhaustColor; //Green, Pink, Red, Yellow, Orange, Purple, Blue, White
    private int speed = 2; //higher = slower
    private int cur = 1;
    private int timer = -1;
    private int startupTimer = 3;    //slowly grow wider
    private int delay = 3;
    private int startupTimerDelay = delay; //make the slowing slower
    
    /**
     * Act - do whatever the Exhaust wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        //Check for game over
        if ( ! getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld()) ) return;
        
        
        timer ++;
        // Animate the exhaust
        if (timer == 0 ) {
            cur = 1;
            if (speed > 15) {startUp();}
        } else if (timer == speed) {
            cur = 2;
            if (speed > 15) {startUp();}
        } else if (timer == speed*2) {
            cur = 3;
            if (speed > 15) {startUp();}
        } else if (timer == speed*3) {
            cur = 4;
            timer = 0 - speed;
            startUp();
        }
        
        setImage(String.format("anim/PNG_Parts&Spriter_Animation/%s/Exhaust/%s/Exhaust%s/exhaust%s.png", decodeColor(color), decodeBoost(boost), startupTimer, cur));
    }
    /**
     * This method is called by the Greenfoot system when this actor has been inserted into the world.
     */
    protected void addedToWorld(World world) {
        setImage(String.format("anim/PNG_Parts&Spriter_Animation/%s/Exhaust/%s/Exhaust%s/exhaust%s.png", decodeColor(color), decodeBoost(boost), startupTimer, cur));
    }
    /**
     * Exhaust start up/ramp down animation
     */
    public void startUp() {
        if (engine) {
            // If the engine is on count down
            if (startupTimerDelay > 0) {
                startupTimerDelay--;
            } else {
                // If fully counted down count startupTimer down (grow one wider)
                startupTimerDelay = delay;
                if (startupTimer > 1) {startupTimer--;}
            }
        } else {
            // If the engine is off count down
            if (startupTimerDelay > 0) {
                startupTimerDelay--;
            } else {
                // If fully counted down count startupTimer up (grow one thinner)
                startupTimerDelay = delay;
                if (startupTimer < 3) {startupTimer++;}
            }
        }
    }
    /**
     * Decode a color String into filename
     */
    public String decodeColor(String color){
        if (color.equals("Green")) {return "Ship1";}
        if (color.equals("Pink")) {return "Ship2";}
        if (color.equals("Red")) {return "Ship3";}
        if (color.equals("Yellow")) {return "Ship4";}
        if (color.equals("Orange")) {return "Ship5";}
        if (color.equals("Purple")) {return "Ship6";}
        if (color.equals("Blue")) {return "Ship7";}
        if (color.equals("White")) {return "Ship8";}
        return "Ship5";
    }
    /**
     * Decode a boost state into filename
     */
    public String decodeBoost(boolean b) {
        if (b) {return "Turbo_flight";} else {return "Normal_flight";}
    }
    /**
     * Decode a width String into filename
     */
    public String decodeState(String state){
        if (state.equals("thin")) {return "1";}
        if (state.equals("normal")) {return "2";}
        if (state.equals("wide")) {return "3";}
        return "3";
    }
    
    /**
     * Set if the Exhaust boosts or not
     */
    public void setBoost(boolean b){boost = b;}
    /**
     * Get if the Exhaust boosts or not
     */
    public boolean getBoost(){return boost;}
    
    /**
     * Set if the Exhaust is on or not
     */
    public void setEngine(boolean e){engine = e;}
    /**
     * Get if the Exhaust is on or not
     */
    public boolean getEngine(){return engine;}
    
    /**
     * Set the Exhaust color (Green, Pink, Red, Yellow, Orange, Purple, Blue, White)
     */
    public void setColor(String c){color = c;}
    /**
     * Get the Exhaust color (Green, Pink, Red, Yellow, Orange, Purple, Blue, White)
     */
    public String getColor(){return color;}
    
    /**
     * Set the Exhaust speed. Higher is slower
     */
    public void setSpeed(int s){speed = s;}
    /**
     * Get the Exhaust speed. Higher is slower
     */
    public int getSpeed(){return speed;}
    
    /**
     * Set the Exhaust rampUp speed. Higher is slower
     */
    public void setStartSpeed(int s){delay = s;}
    /**
     * Get the Exhaust rampUp speed. Higher is slower
     */
    public int getStartSpeed(){return delay;}
}
