import greenfoot.*;
import java.util.ArrayList;
import java.util.Arrays;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;
import java.util.List;

/**
 * HelperClass
 * Provides handy tools
 * 
 * @author Greenfoot + Jannis Martensen
 * @version 1.0
 */
public class HelperClass extends Collision
{
    private double exactX;
    private double exactY;
    /**
     * Resize an image resource
     * Author Jannis Martensen
     */
    public void imageSize(int size) {
        GreenfootImage image = getImage();
        image.scale(image.getWidth() + size, image.getHeight() + size);
        setImage(image);
    }
    /**
     * Generate a boolean value based
     * on the parameter chance.
     * Author Jannis Martensen
     */
    public static boolean chance(int chance)
    {
        if (chance > 100) {
            chance = 100;
        } else if (chance < 0) {
            chance = 0;
        }
        return Greenfoot.getRandomNumber(100) < chance;
    }
    /**
     * Generate a boolean value based
     * on the parameter chance.
     * Author Jannis Martensen
     */
    public static boolean chance(double chance)
    {
        if (chance > 100.0) {
            chance = 100.0;
        } else if (chance < 0.0) {
            chance = 0.0;
        }
        return Greenfoot.getRandomNumber(10000) < chance*100;
    }  
    /**
     * If touching PlayerShots subtract one health point, if none left remove itself
     * Author Jannis Martensen
     */
    public ArrayList<Integer> touchPlayerShot(int time, int health, int Points) {
        if (touchesPlayerShot()) {
            if (time == 1) {
                time --;
            } else if(time == 0) {
                if (health > 0) {
                    health --;
                    if (health == 0) {
                        Spaceship.Points += Points;
                        getWorld().removeObject(this);
                        return new ArrayList<>(Arrays.asList(time, health));
                    }
                }
                time = 40;
            }
        }
        return new ArrayList<>(Arrays.asList(time, health));
    }
    /**
     * If touching PlayerShots subtract one health point, if none left remove itself, exhaust and healthbar
     * Author Jannis Martensen
     */
    public ArrayList<Integer> touchPlayerShot(int time, int health, Exhaust exhaust, Bar bar, int Points) {
        if (touchesPlayerShot()) {
            if (time == 1) {
                time --;
            } else if(time == 0) {
                if (health > 0) {
                    health --;
                    if (health == 0) {
                        Spaceship.Points += Points;
                        getWorld().removeObject(exhaust);
                        getWorld().removeObject(bar);
                        getWorld().removeObject(this);
                        return new ArrayList<>(Arrays.asList(time, health));
                    }
                }
                time = 40;
            }
        }
        return new ArrayList<>(Arrays.asList(time, health));
    }
    /**
     * If touching EnemyShots subtract one health point, if none left remove itself
     * Author Jannis Martensen
     */
    public ArrayList<Integer> touchAnyShot(int time, int health, int Points) {
        if (touch(Shots.class)) {
            if (time == 1) {
                time --;
            } else if(time == 0) {
                if (health > 0) {
                    health --;
                    if (health == 0) {
                        Spaceship.Points += Points;
                        getWorld().removeObject(this);
                        return new ArrayList<>(Arrays.asList(time, health));
                    }
                }
                time = 40;
            }
        }
        return new ArrayList<>(Arrays.asList(time, health));
    }
    public boolean touchesEnemyShot() {
        Shots s = (Shots) getOneTouchedObject(Shots.class);
        if (s == null) {
            return false;
        }
        return s.getType() == 2;
    }
    public boolean touchesPlayerShot() {
        Shots s = (Shots) getOneTouchedObject(Shots.class);
        if (s == null) {
            return false;
        }
        return s.getType() == 1;
    }
    /**
     * Move forward by the specified distance.
     * (Overrides the method in Actor).
     */
    @Override
    public void move(int distance)
    {
        move((double)distance);
    }
    /**
     * Move sideways by the specified distance.
     * Author Jannis Martensen
     */
    public void moveSide(int distance)
    {
        moveSide((double)distance);
    }
    /**
     * Move forward by the specified exact distance.
     */
    public void move(double distance)
    {
        double radians = Math.toRadians(getRotation());
        double dx = Math.cos(radians) * distance;
        double dy = Math.sin(radians) * distance;
        try {
            if (exactX + dx > getWorld().getWidth()+1) {
                dx = 0;
            }
            if (exactY + dy > getWorld().getHeight()+1) {
                dy = 0;
            }
            if (exactX + dx < -1) {
                dx = 0;
            }
            if (exactY + dy < -1) {
                dy = 0;
            }
        } catch (NullPointerException e) {
            // ignore
        }
        setLocation(exactX + dx, exactY + dy);
    }
    /**
     * Move sideways by the specified distance.
     * Author Jannis Martensen
     */
    public void moveSide(double distance)
    {
        double radians = Math.toRadians(getRotation()+90);
        double dx = Math.cos(radians) * distance;
        double dy = Math.sin(radians) * distance;
        try {
            if (exactX + dx > getWorld().getWidth()+1) {
                dx = 0;
            }
            if (exactY + dy > getWorld().getHeight()+1) {
                dy = 0;
            }
            if (exactX + dx < -1) {
                dx = 0;
            }
            if (exactY + dy < -1) {
                dy = 0;
            }
        } catch (NullPointerException e) {
            // ignore
        }
        setLocation(exactX + dx, exactY + dy);
    }
    /**
     * Set the location using exact coordinates.
     */
    public void setLocation(double x, double y) 
    {
        exactX = x;
        exactY = y;
        super.setLocation((int) (x + 0.5), (int) (y + 0.5));
    }
    /**
     * Set the location using integer coordinates.
     * (Overrides the method in Actor.)
     */
    @Override
    public void setLocation(int x, int y) 
    {
        exactX = x;
        exactY = y;
        super.setLocation(x, y);
    }
    /**
     * Return the exact x-coordinate (as a double).
     */
    public double getExactX() 
    {
        return exactX;
    }
    /**
     * Return the exact y-coordinate (as a double).
     */
    public double getExactY() 
    {
        return exactY;
    }
}
