import greenfoot.*;
import java.util.ArrayList;

/**
 * NewLevelTemplate
 * Template for creating a new level.
 * Just copy this and then change your values to add a new level.
 * You will need to add the buttons in the Title Menu yourself though.
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class Level extends WorldHelper
{
    private int Counter;
    private boolean added = false;
    private Label speed;
    /*
     * Level Settings: Set the enemy spawn rate and level name here
     */
    private int maxAsteroids = 15;
    private int maxEasyEnemies = 7;
    private int maxAttackerEnemies = 0;
    private int maxBossEnemies = 0;
    
    
    
    // This name will be shown in the Scoreboard and the Level Title on the top left
    private String levelName = "Level";
    private String levelNameGerman = "Level";
    /**
     * Constructor
     */
    public Level(int asteroids, int easyEnemies, int attackerEnemies, int bossEnemies, String lName, String lNameGerman) 
    {
        maxAsteroids = asteroids;
        maxEasyEnemies = easyEnemies;
        maxAttackerEnemies = attackerEnemies;
        maxBossEnemies = bossEnemies;
        levelName = lName;
        levelNameGerman = lNameGerman;
        
        
        if (Settings.Language.equals("en")) {
            Label level = new Label(String.format("Level: %s", levelName), 50);
            addObject(level, getWidth()/2, 40);
        } else if (Settings.Language.equals("de")) {
            Label level = new Label(String.format("Level: %s", levelNameGerman), 50);
            addObject(level, getWidth()/2, 40);
        }
        
        speed = new Label("", 50);
        addObject(speed, getWidth()/2, getHeight()-50);
        
        // Spawn spaceship
        if (Settings.TwoPlayerMode) {
            addObject(new Spaceship(1), (int)((double)getWidth()/3.0 * 1.29)-4, getHeight()/2);
            addObject(new Spaceship(2), (int)((double)getWidth()/3.0 * 1.71)-4, getHeight()/2+3);
        } else {
            addObject(new Spaceship(1), getWidth()/2, getHeight()/2);
        }
    }
    /**
     * Get the name of the current world for level unlock
     */
    public String getName() {return levelName;}
    /**
     * Act - do whatever the Level wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act(){
        //Ceck for pause/game over
        if ( ! getObjects(Button.class).isEmpty() ) {
            // HashMap<String, ArrayList<Integer>>
            if (!Settings.ScoreList.containsKey(levelName) && !added) {
                added = true;
                ArrayList<Integer> l = new ArrayList();
                l.add(Spaceship.Points);
                Settings.ScoreList.put(levelName, l);
            } else if (!added) {
                added = true;
                ArrayList l = Settings.ScoreList.get(levelName);
                l.add(Spaceship.Points);
                Settings.ScoreList.put(levelName, l);
            }
        }
        IncreaseEnemySpeed(speed);
        Asteroids(Counter, maxAsteroids);
        EasyEnemies(maxEasyEnemies);
        AttackerEnemies(maxAttackerEnemies);
        BossEnemies(maxBossEnemies);
    }
}