import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.util.List;
import java.lang.reflect.*;
import javax.xml.transform.OutputKeys;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import greenfoot.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

/**
 * @author Jannis Martensen
 * @version 1.0
 */
public class SaveFiles {
 
    //public static String xmlFilePath = "GameSave.xml";
 
    /** 
     * SaveLoad game save to file
     * @param FileName: savename
     */
    public static void loadSave(String FileName) {
        try {
            // If file does not exist stop
            File inputFile = new File("Saves/" + FileName + ".save");
            if (!inputFile.exists()) {
                return;
            }
            
            // Start DocumentBuilder for XML and parse the input file
            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            Document doc = dBuilder.parse(inputFile);
            doc.getDocumentElement().normalize();

            // Get root element
            Element root = doc.getDocumentElement();
            
            // Get all child nodes containing the individual Settings
            NodeList nList = root.getChildNodes();
            
            Field[] fields = Settings.class.getFields();
            
            // Get already running instance of Settings.class
            Settings s = Settings.getInstance();
            
            for (int i = 0; i < nList.getLength(); i++) {
                Node node = nList.item(i);
                // If node is relevant
                if (node.getNodeType() == Node.ELEMENT_NODE) {
                    try {
                        // Assign values
                        Field f = Settings.class.getField(node.getNodeName());
                        f.setAccessible(true);
                        if (f.getType() == int.class) {
                            f.set(s, Integer.parseInt(node.getTextContent()));
                        } else if (f.getType() == double.class) {
                            f.set(s, Double.parseDouble(node.getTextContent()));
                        } else if (f.getType() == String.class) {
                            f.set(s, String.valueOf(node.getTextContent()));
                        } else if (f.getType() == boolean.class) {
                            f.set(s, Boolean.parseBoolean(node.getTextContent()));
                        } else if (f.getType() == HashMap.class) {
                            
                            String value = node.getTextContent().substring(1, node.getTextContent().length()-1); //remove brackets
                            String[] keyValuePairs = value.split("],");              //split the string to creat key-value pairs
                            HashMap<String,ArrayList<Integer>> map = new HashMap<>();               
                            
                            for(String pair : keyValuePairs)                        //iterate over the pairs
                            {
                                String[] entry = pair.split("=");                   //split the pairs to get key and value
                                ArrayList<Integer> n = new ArrayList();
                                
                                // Get the score list
                                List keys = Arrays.asList(entry[1].toString().trim().replace("[","").replace("]",""));
                                List<String> l = new ArrayList<String>();
                                
                                // Add to arraylist
                                for (String str2 : entry[1].trim().replace("[","").replace("]","").split(",")) {
                                    n.add(Integer.parseInt(str2.trim()));
                                }
                                map.put(entry[0].trim(), n);          //add them to the hashmap and trim whitespaces
                            }
                                                        
                            f.set(s, map);
                        }
                        
                    } catch (NoSuchFieldException ex) {
                        // If field not found
                        // ex.printStackTrace();
                        /* While relying on exceptions is bad practice, there seems to be no other way in this case */
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /** 
     * Save a game save to file
     * @param FileName: savename
     */
    public static void saveSave(String FileName) {
        Element v;
        try {
            File inputFile = new File("Saves/" + FileName + ".save");
            
            // Create savefile if non-existing
            try{inputFile.createNewFile();}
            catch (java.io.IOException ioe){ioe.printStackTrace();}
            
            Field[] fields = Settings.class.getFields();
            
            // Start DocumentBuilder for XML
            DocumentBuilderFactory documentFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = documentFactory.newDocumentBuilder();
            Document document = documentBuilder.newDocument();
 
            // root element
            Element root = document.createElement("GameSettings");
            document.appendChild(root);
            
            Settings s = Settings.getInstance();
            
            // For each setting get the name and value and write it to the xml file
            for(int i=0;i<fields.length;i++){
                // element
                if (fields[i].getType() != ArrayList.class) {
                    v = document.createElement(String.valueOf(fields[i].getName()));
                    v.appendChild(document.createTextNode(String.valueOf(fields[i].get(s))));
                    root.appendChild(v);
                }
            }
 
            
            // create the xml file
            //transform the DOM Object to an XML File
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            DOMSource domSource = new DOMSource(document);
            StreamResult streamResult = new StreamResult(inputFile);
 
            transformer.transform(domSource, streamResult);
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        } catch (TransformerException tfe) {
            tfe.printStackTrace();
        } catch (IllegalAccessException iae) {
            iae.printStackTrace();
        }
    }
}
