import greenfoot.*;
import java.util.ArrayList;
import java.io.*;
import java.util.Arrays;

/**
 * SaveFileSelector
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class SaveSelector extends WorldHelper
{
    private Scroller scroller;
    Label level;
    Button settings;
    Image t;
    ArrayList<File> filesList = new ArrayList<>();
    ArrayList<Button> buttonList = new ArrayList<>();
    int StartM = 2;
    int m = StartM;
    int distance = 75;
    boolean load = false;
    Button customEntry = null;
    ArrayList<Object> a = new ArrayList<>();
        
        
    
    /**
     * @param l: load mode or not (save mode)
     */
    public SaveSelector(boolean l)
    {
        
        load = l;
        // get all files
        listFiles("Saves");
        
        // Display scroll key help
        t = new Image();
        t.setImage("anleitungScroll.png");
        addObject(t, getWidth()/15, getHeight()/2);
        
        // Title and back button
        if (Settings.Language.equals("en")) {
            level = new Label("Select a Save", 50);
            addObject(level, getWidth()/2, 40);
            
            settings = new Button("Back", 1);
            addObject(settings, getWidth()-120, getHeight()-50);
        } else if (Settings.Language.equals("de")) {
            level = new Label("Wähle einen Spielstand", 50);
            addObject(level, getWidth()/2, 40);
            
            settings = new Button("Zurück", 1);
            addObject(settings, getWidth()-120, getHeight()-50);
        }
        
        // If in save mode add a new save option
        if (!load) {
            StartM ++;
            if (Settings.Language.equals("en")) {
                customEntry = new Button("New Save...", 1);
                customEntry.setRounded(false);
                a.add(customEntry);
                addObject(customEntry, (int)(getWidth()/2), (int)(distance*m));
            } else if (Settings.Language.equals("de")) {
                customEntry = new Button("Neuer Speicherstand", 1);
                customEntry.setRounded(false);
                a.add(customEntry);
                addObject(customEntry, (int)(getWidth()/2), (int)(distance*m));
            }
            m++;
        }
        int d;
        
        // Set worldheight based on list length
        if ((filesList.size()+StartM)*distance > getHeight()) {d = (filesList.size()+StartM+1)*distance;} else {d = getHeight();}
        
        // Scroller class
        scroller = new Scroller(this, new GreenfootImage("background.jpg"), getWidth(), d);
        
        // Add the buttons for all save files
        for (File file : filesList) {
            Button b = new Button(file.toString().replace("Saves\\", "").replace(".save", ""), 1);
            b.setRounded(false);
            buttonList.add(b);
            a.add(b);
            addObject(b, (int)(getWidth()/2), (int)(distance*m));
            m++;
        }
       
        m = StartM;
    }
    
    /**
     * Handles scrolling, showing/hiding objects and pressing buttons
     */
    public void act()
    {
        // Navigate back to the Title Menu
        if (Greenfoot.mouseClicked(settings) || Greenfoot.isKeyDown("escape")) {
            Greenfoot.delay(50);
            Greenfoot.setWorld(new SettingsScreen());
        }
        
        
        
        // Scroll world
        if (Greenfoot.isKeyDown("up")) {
            scrollUp();
        } else if(Greenfoot.isKeyDown("down")) {
            scrollDown();
        }
        
        
        // New Save Button
        if (customEntry != null) {
            // OnClick ask for new file name and then save to the file
            if (Greenfoot.mouseClicked(customEntry)) {
                String input = "GameSave";
                if (Settings.Language.equals("en")) {
                    input = Greenfoot.ask("Please enter the name of the save: ");
                } else if (Settings.Language.equals("de")) {
                    input = Greenfoot.ask("Bitte gebe den Namen des Speicherstandes ein: ");
                }
                SaveFiles.saveSave(input);
                Greenfoot.setWorld(new SettingsScreen());
            }
            int threshTop = 100;// custom threshold to not collide with the top Text
            int ButtonPosition = (StartM -1) * distance;
            int scrollDistance = scroller.getScrolledY() + threshTop; 
            int scrollScreenEnd = scrollDistance-threshTop+getHeight();
            
            // If Buttons outside of scroll, hide it
            if ((scrollDistance >= ButtonPosition || ButtonPosition >= scrollScreenEnd) && customEntry.getImage().getTransparency() != 0) {
                GreenfootImage image = customEntry.getImage();
                image.setTransparency(0);
                customEntry.setImage(image);
                
            } else if (scrollDistance < ButtonPosition && ButtonPosition < scrollScreenEnd && customEntry.getImage().getTransparency() == 0) {
                // If the button position is visible in current scrollscreen, show and correctly position it
                
                // If button position is nearer to the bottom spawn at the bottom and spawn at the top if closer to the top
                if (scrollScreenEnd - ButtonPosition < ButtonPosition - scrollDistance) {
                    // If the button position is closer to the top or bottom
                    customEntry.setLocation((int)(getWidth()/2), getHeight()-1); 
                } else {
                    customEntry.setLocation((int)(getWidth()/2), 1+threshTop);   
                }
                GreenfootImage image = customEntry.getImage();
                image.setTransparency(200);
                customEntry.setImage(image);
            }
        }
        
        // Keep static elements in the same position
        // level.setLocation(getWidth()/2, 40);
        // settings.setLocation(getWidth()-120, getHeight()-50);
        // t.setLocation(getWidth()/15, getHeight()/2);
        
        // Iterate through all buttons and get click state and set visibility according to position
        for (int i = 0; i < buttonList.size(); i++) {
            Button button = buttonList.get(i);
            int threshTop = 100;// custom threshold to not collide with the top Text
            int ButtonPosition = (StartM + i) * distance;
            int scrollDistance = scroller.getScrolledY() + threshTop; 
            int scrollScreenEnd = scrollDistance-threshTop+getHeight();
            
            // If Buttons outside of scroll, hide it
            if ((scrollDistance >= ButtonPosition || ButtonPosition >= scrollScreenEnd) && button.getImage().getTransparency() != 0) {
                GreenfootImage image = button.getImage();
                image.setTransparency(0);
                button.setImage(image);
                
            } else if (scrollDistance < ButtonPosition && ButtonPosition < scrollScreenEnd && button.getImage().getTransparency() == 0) {
                // If the button position is visible in current scrollscreen, show and correctly position it
                
                // If button position is nearer to the bottom spawn at the bottom and spawn at the top if closer to the top
                if (scrollScreenEnd - ButtonPosition < ButtonPosition - scrollDistance) {
                    // If the button position is closer to the top or bottom
                    button.setLocation((int)(getWidth()/2), getHeight()-1); 
                } else {
                    button.setLocation((int)(getWidth()/2), 1+threshTop);   
                }
                GreenfootImage image = button.getImage();
                image.setTransparency(200);
                button.setImage(image);
            }
            
            // Depending on mode, perform either load or save
            if (Greenfoot.mouseClicked(button)) {
                if (load) {
                    SaveFiles.loadSave(button.getText());
                    Greenfoot.delay(50);
                    Greenfoot.setWorld(new TitleScreen());
                } else {
                    SaveFiles.saveSave(button.getText());
                    Greenfoot.delay(50);
                    Greenfoot.setWorld(new TitleScreen());
                }
            }
        }
    }
    private void scrollDown() {
        scroller.scroll(0, 2, a, true);
    }
    private void scrollUp() {
        scroller.scroll(0, -2, a, true);
    }
    public void listFiles(String directoryName) {
        File directory = new File(directoryName);

        // Get all files from a directory.
        ArrayList<File> fList = new ArrayList<>(Arrays.asList(directory.listFiles()));
        if(fList != null) {
            for (File file : fList) {
                if (file.isFile()) {
                    // only get .save files
                    String ending = file.toString().trim().split("\\.")[file.toString().trim().split("\\.").length -1].toLowerCase();
                    if ("save".equals(ending)) {
                        filesList.add(file);
                    }
                } else if (file.isDirectory()) {
                    // Also search all subdirectories
                    listFiles(file.getAbsolutePath());
                }
            }
        }
    }
    
    
}
