import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.Arrays;
import java.util.ArrayList;

/**
 * SettingsScreen
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class SettingsScreen extends WorldHelper
{
    public Button settings;
    public Button save;
    public Button twoPlayer;
    public Label out;
    public Button load;
    private Button selectLanguage;
    private int Counter = 0;
    
    /**
     * Constructor for objects of class SettingsScreen.
     */
    public SettingsScreen()
    {    
        setWorldColor(15, 15, 15, 255);
        
        // Add the Setting Label and the back Button
        if (Settings.Language.equals("en")) {
            Label level = new Label("Settings", 50);
            addObject(level, getWidth()/2, 40);
            
            addObject(new Label("Game Saves:", 50), (int)(getWidth()/(double)8), (int)(getHeight()/5.0));
            
            settings = new Button("Back", 1);
            addObject(settings, getWidth()-120, getHeight()-50);
            
            addObject(new Label("Two Player Mode:", 50), (int)(getWidth()/(double)8), (int)(getHeight()/5.0*1.5));
            twoPlayer = new Button("OFF", 1);
            addObject(twoPlayer, (int)(getWidth()/8.0)*7, (int)(getHeight()/5.0*1.5));
            
            save = new Button("Save", 1);
            addObject(save, (int)(getWidth()/8.0)*7, (int)(getHeight()/5.0));
            
            load = new Button("Load", 1);
            addObject(load, (int)(getWidth()/8.0)*6, (int)(getHeight()/5.0));
            
        } else if (Settings.Language.equals("de")) {
            Label level = new Label("Einstellungen", 50);
            addObject(level, getWidth()/2, 40);
            
            addObject(new Label("Spielstände:", 50), (int)(getWidth()/8.0), (int)(getHeight()/5.0));
            
            settings = new Button("Zurück", 1);
            addObject(settings, getWidth()-120, getHeight()-50);
            
            addObject(new Label("Zwei Spieler Modus:", 50), (int)(getWidth()/(double)8), (int)(getHeight()/5.0*1.5));
            twoPlayer = new Button("AUS", 1);
            addObject(twoPlayer, (int)(getWidth()/8.0)*7, (int)(getHeight()/5.0*1.5));
            
            save = new Button("Speichern", 1);
            addObject(save, (int)(getWidth()/8.0)*7, (int)(getHeight()/5.0));
            
            load = new Button("Laden", 1);
            addObject(load, (int)(getWidth()/8.0)*6, (int)(getHeight()/5.0));
            
        }
        
        if (!Settings.TwoPlayerMode) {
            if (Settings.Language.equals("de")) {twoPlayer.setText("AUS");
            } else if (Settings.Language.equals("en")) {twoPlayer.setText("OFF");}
        } else if (Settings.TwoPlayerMode) {
            if (Settings.Language.equals("de")) {twoPlayer.setText("AN");
            } else if (Settings.Language.equals("en")) {twoPlayer.setText("ON");}
        }
        
        out = new Label("", 50);
        addObject(out, (int)(getWidth()/8.0)*4, (int)(getHeight()/5.0));
        
        // Language Select Button
        selectLanguage = new Button(Settings.Language.toUpperCase(), 1);
        addObject(selectLanguage, 80, getHeight()-50);
        
    }
    /**
     * Act - do whatever the Level Asteroids wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act(){
        
        // Navigate back to the Title Menu
        if (Greenfoot.mouseClicked(settings) || Greenfoot.isKeyDown("escape")) {
            Greenfoot.delay(50);
            Greenfoot.setWorld(new TitleScreen());
        }
        
        if (Greenfoot.mouseClicked(load)) {
            Greenfoot.setWorld(new SaveSelector(true));
        } else if (Greenfoot.mouseClicked(save)) {
            Greenfoot.setWorld(new SaveSelector(false));
        }
        
        // Change the current Language
        if (Greenfoot.mouseClicked(selectLanguage) && Settings.Language.equals("en")) {
            selectLanguage.setText("DE");
            Settings.Language = "de";
            Greenfoot.setWorld(new SettingsScreen());
        } else if (Greenfoot.mouseClicked(selectLanguage) && Settings.Language.equals("de")) {
            selectLanguage.setText("EN");
            Settings.Language = "en";
            Greenfoot.setWorld(new SettingsScreen());
        }
        
        // Change the player mode
        if (Greenfoot.mouseClicked(twoPlayer) && Settings.TwoPlayerMode) {
            if (Settings.Language.equals("de")) {twoPlayer.setText("AUS");
            } else if (Settings.Language.equals("en")) {twoPlayer.setText("OFF");}
            
            Settings.TwoPlayerMode = false;
        } else if (Greenfoot.mouseClicked(twoPlayer) && !Settings.TwoPlayerMode) {
            if (Settings.Language.equals("de")) {twoPlayer.setText("AN");
            } else if (Settings.Language.equals("en")) {twoPlayer.setText("ON");}
            
            Settings.TwoPlayerMode = true;
        }
    }
}
