import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.io.FileNotFoundException;

/**
 * Shots
 * The Shots beeing fired by the User and the Enemies
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class Shots extends HelperClass
{
    private int timer = 0;
    private String color = "Orange"; //Green, Pink, Red, Yellow, Orange, Purple
    private Label label;
    private int firstStepSize = 3;
    private int secondStepSize = 4;
    private int scale;
    private boolean end = false;
    private int NormalScale = 25;
    private int MediumScale = 75;
    private int BigScale = 150;
    private boolean explode = false;
    private boolean firedByEnemy = false;
    int type = 1;
    
    
    /**
     * Constructor for objects of class Shots.
     * Rotation: Set spawn rotation of the shot
     * instantExplode: instantly explode upon spawn, no matter if it is touching an enemy
     * type: 1: Playershot, 2: Enemyshot
     */
    public Shots(int rotation, boolean instantExplode, int t) {
        if (t != 1 && t != 2) {
            t = 1;
        }
        type = t;
        
        
        explode = instantExplode;
        if (instantExplode) {
            timer = firstStepSize*7 + secondStepSize;
        }
        setRotation(rotation);
        scale = NormalScale;
        setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_1.png", decodeColor(color), decodeColor(color)));
        
    }
    /**
     * Get the type of shot
     * 1: Playershot, 2: Enemyshot
     */
    public int getType() {
        return type;
    }
    /**
     * Act - do whatever the shots wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        //Ceck for game over
        if ( ! getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld()) ) return;
        
        // If touchChecker start explosion animation and set scale accoridngly to asteroid size
        if (touchChecker(timer, firstStepSize)) {
            timer = firstStepSize*7 + secondStepSize;
            Asteroid s = (Asteroid) getOneTouchedObject(Asteroid.class);
            if (s != null) {
                switch (s.getType()) {
                    case 2:  
                        scale = MediumScale;
                    case 3:
                        scale = BigScale;
                    default:
                        scale = NormalScale;
                }
            }
        }
    
        // Move unless the explosion animation started
        if (timer == firstStepSize*6 && !explode) {
            move(Settings.ShotsSpeed);
        }
        // Count animation timer up if in shot or explosion animation, otherwise wait
        if (timer < firstStepSize*6 || timer > firstStepSize*6) {
            timer++;
        }
        
        // Use custom isAtEdge method because of detection issues
        if (getX() >= getWorld().getWidth() -5
        ||  getY() >= getWorld().getHeight() -5
        ||  getX() <= 5
        ||  getY() <= 5) {
            getWorld().removeObject(this);
            return;
        }
        
        // Shot animation
        if (timer == 0) {    
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_1.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_2.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize*2) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_3.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize*3) {
            try {
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_4.png", decodeColor(color), decodeColor(color)));
            } catch(Exception e){
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_asset.png", decodeColor(color), decodeColor(color)));
            }
            imageSize(scale);
        } else if (timer == firstStepSize*4) {
            try {
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_5.png", decodeColor(color), decodeColor(color)));
            } catch(Exception e){
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_asset.png", decodeColor(color), decodeColor(color)));
            }
            imageSize(scale);
        } else if (timer == firstStepSize*5) {
            try {
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_6.png", decodeColor(color), decodeColor(color)));
            } catch(Exception e){
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_asset.png", decodeColor(color), decodeColor(color)));
            }
        } else if (timer == firstStepSize*6) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_asset.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
            
        } else if (timer == firstStepSize*7 + secondStepSize) {
            // Explosion Animation
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_exp1.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
            if(touch(Asteroid.class)) {getWorld().removeObject(this);return;}
        } else if (timer == firstStepSize*7 + secondStepSize*2) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_exp2.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*3) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_exp3.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*4) {
            setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_exp4.png", decodeColor(color), decodeColor(color)));
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*5) {
            check("5");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*6) {
            check("6");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*7) {
            check("7");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*8) {
            check("8");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*9) {
            check("9");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*10) {
            check("10");
            imageSize(scale);
        } else if (timer == firstStepSize*7 + secondStepSize*11) {
            getWorld().removeObject(this);
        } 
    }
    /**
     * Check if the shot touches an Enemy or an Asteroid
     */
    public boolean touchChecker(int timer, int firstStepSize){
        if (type == 1) {
            return (touch(Enemies.class) || touch(Asteroid.class)) && timer <= firstStepSize*6;
        } else {
            return (touch(Spaceship.class) || touch(Asteroid.class)) && timer <= firstStepSize*6;
        }
    }
    /**
     * Check if the image resource is available, otherwise stop the animation
     */
    public void check(String n) {
        try {
                setImage(String.format("anim/PNG_Parts&Spriter_Animation/Shots/Shot%s/shot%s_exp%s.png", decodeColor(color), decodeColor(color), n));
        } catch(Exception e){
            if (end) {
                getWorld().removeObject(this);
            }
            end = true;
        }
    }
    /**
     * Set the Shots color (Green, Pink, Red, Yellow, Orange, Purple)
     */
    public void setColor(String c){color = c;}
    /**
     * Get the Shots color (Green, Pink, Red, Yellow, Orange, Purple)
     */
    public String getColor(){return color;}
    /**
     * decode a color String into filename
     */
    public String decodeColor(String color){
        if (color.equals("Green")) {return "1";}
        if (color.equals("Pink")) {return "2";}
        if (color.equals("Red")) {return "3";}
        if (color.equals("Yellow")) {return "5";}
        if (color.equals("Orange")) {return "4";}
        if (color.equals("Purple")) {return "6";}
        return "Ship5";
    }
    /**
     * This method is called by the Greenfoot system when this actor has been inserted into the world.
     */
    protected void addedToWorld​(World world) {
        imageSize(scale);
    }
}
