
import greenfoot.*;

/**
 * Spaceship
 * This is the Spaceship that will be controlled by the user
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class Spaceship extends HelperClass
{
    public static int Points = 0;
    private Label label;
    private Label pName;
    private Image t;
    private Button home;
    private Button retry;
    private Darkener d;
    private Image nl;
    private Button backToGame;
    private Bar counter;
    private Bar BoostBar;
    private Exhaust exhaust;
    private int timer = 0;
    private double health = 100.0;
    private int escTimer;
    private int damageTimer = 0;
    private int imageTimer = 0;
    private double bTimer = 0;
    private int acceleration = 0;
    private boolean recharge = false;
    private boolean boost = false;
    private boolean dmg = false;
    private boolean shownMessage = false;
    private boolean menuOpen = false;
    private String playerName;
    private int playerNumber;
    private String forwardKey;
    private String backwardKey;
    private String leftKey;
    private String rightKey;
    private String boostKey;
    private String shootKey;
    private String scColor;
    private String ecColor;
    private String soColor;
    
    public Spaceship(int pp) {
        playerName = String.format("P%s", pp);
        playerNumber = pp;
        
        if (pp == 1) {
            forwardKey = "W";
            backwardKey = "S";
            leftKey = "A";
            rightKey = "D";
            boostKey = "shift";
            shootKey = "space";
            
            scColor = Settings.SpaceshipColor;
            ecColor = Settings.SpaceshipExhaustColor;
            soColor = Settings.SpaceshipShotsColor;
        } else {
            forwardKey = "I";
            backwardKey = "K";
            leftKey = "J";
            rightKey = "L";
            boostKey = "#";
            shootKey = "enter";
            
            scColor = Settings.Spaceship2Color;
            ecColor = Settings.Spaceship2ExhaustColor;
            soColor = Settings.Spaceship2ShotsColor;
        }
    }
    /**
     * Act - do whatever the User wants the Spaceship to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        // Only do these for one player
        if(playerNumber == 1 || (playerNumber == 2 && getWorld().getObjects(Spaceship.class).size() == 1)) {
            // If the back to menu button is pressed, the button exists and not in Title Screen, go back to title menu
            if (Greenfoot.mouseClicked(home) && !getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld())){
                Points = 0;
                Greenfoot.setWorld(new TitleScreen());
                getWorld().setPaintOrder();
            }
            // If the retry button is pressed, the button exists and not in Title Screen, restart the level
            if (Greenfoot.mouseClicked(retry) && !getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld())){
                World world = null;
                try { world = getWorld().getClass().newInstance(); } catch (Exception e) { }
                if (world != null) {Greenfoot.setWorld(world); } else {Greenfoot.setWorld(new TitleScreen());}
                Points = 0;
                getWorld().setPaintOrder();
            }
            // If the back to game button is pressed, the button exists and not in Title Screen, exit the pause menu
            if (Greenfoot.mouseClicked(backToGame) && menuOpen && !getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld())){
                menuOpen = false;
                getWorld().removeObject(home);
                getWorld().removeObject(retry);
                getWorld().removeObject(d);
                getWorld().removeObject(backToGame);
                getWorld().setPaintOrder();
            }
            if (escTimer > 0) {
                escTimer --;
            }
            // If "escape" is pressed, the escTimer is 0 again, health is not 0 (game over) and not in titlescreen show the pause menu
            if(Greenfoot.isKeyDown("escape") && escTimer == 0 && health != 0 && !TitleScreen.class.isInstance(getWorld())) {
                if (!menuOpen) {
                    getWorld().setPaintOrder(Button.class, Label.class, Image.class, Darkener.class, Shots.class, Asteroid.class, Bar.class, Enemies.class, Exhaust.class, Spaceship.class);
                    // Add Darkener and Buttons to pause menu
                    d = new Darkener();
                    getWorld().addObject(d, 1, 1);
                    escTimer = 100;
                    menuOpen = true;
                    
                    if (Settings.Language.equals("en")) {
                        home = new Button("Back to Menu", 1);
                        getWorld().addObject(home, getWorld().getWidth()/2, getWorld().getHeight()/2+90);
                    } else if (Settings.Language.equals("de")) {
                        home = new Button("Zurück zum Hauptmenü", 1);
                        getWorld().addObject(home, getWorld().getWidth()/2, getWorld().getHeight()/2+90);
                    }
                    if (Settings.Language.equals("en")) {
                        retry = new Button("Restart", 1);
                        getWorld().addObject(retry, getWorld().getWidth()/2, getWorld().getHeight()/2);
                    } else if (Settings.Language.equals("de")) {
                        retry = new Button("Neustarten", 1);
                        getWorld().addObject(retry, getWorld().getWidth()/2, getWorld().getHeight()/2);
                    }
                    if (Settings.Language.equals("en")) {
                        backToGame = new Button("Continue", 1);
                        getWorld().addObject(backToGame, getWorld().getWidth()/2, getWorld().getHeight()/2-90);
                    } else if (Settings.Language.equals("de")) {
                        backToGame = new Button("Fortsetzen", 1);
                        getWorld().addObject(backToGame, getWorld().getWidth()/2, getWorld().getHeight()/2-90);
                    }
                } else {
                    // If pressed while the menu is open remove the menu
                    getWorld().setPaintOrder();
                    escTimer = 50;
                    menuOpen = false;
                    getWorld().removeObject(d);
                    getWorld().removeObject(home);
                    getWorld().removeObject(retry);
                    getWorld().removeObject(backToGame);
                }
            }
        }
        
        
        //Check for game over
        if ( ! getWorld().getObjects(Button.class).isEmpty() && !TitleScreen.class.isInstance(getWorld()) ) return;
        
        checkKeyPress();
        
        // If damageTimer has not run out and not in TitleScreen animate damage timout
        if (damageTimer > 0 && !TitleScreen.class.isInstance(getWorld())) {
            damageTimer--;
            if (damageTimer%100 == 0) {
                if (dmg) {
                    dmg = false;
                    setImage(String.format("spaceship(%s).png", scColor.toLowerCase()));
                } else {
                    dmg = true;
                    setImage("spaceship(damage).png");
                }
            }
        }
        
        // If touching anything that hurts the spaceship, not in TitleScreen and the damage timout has run out remove one health point, if none left, game over
        if ((touch(Asteroid.class) || touch(AttackerEnemies.class) || touch(BossEnemies.class) || touchesEnemyShot()) && damageTimer == 0 && !TitleScreen.class.isInstance(getWorld())) {
            // If touching Asteroid spawn one PlayerShot and also damage the Asteroid once
            if (touch(Asteroid.class)) {
                Shots actor = new Shots(getRotation(), true, 1);
                Actor asteroid = getOneTouchedObject(Asteroid.class);
                actor.setColor(Settings.SpaceshipShotsColor);
                getWorld().addObject(actor, asteroid.getX(), asteroid.getY());
            }
            health = health - (100/(double)Settings.SpaceshipHealth);
            if (health <= 0) {health = 0;}
            counter.setValue((int)health);
            damageTimer = 600;
            if (health == 0 && getWorld().getObjects(Spaceship.class).size() == 1 ){
                d = new Darkener();
                getWorld().addObject(d, 1, 1);
                
                t = new Image();
                t.setImage("GameOver.png");
                getWorld().addObject(t, getWorld().getWidth()/2, getWorld().getHeight()/3);
                
                getWorld().setPaintOrder(Button.class, Label.class, Image.class, Darkener.class, Shots.class, Asteroid.class, Bar.class, Enemies.class, Exhaust.class, Spaceship.class);
                if (Settings.Language.equals("en")) {
                    retry = new Button("Retry", 1);
                    getWorld().addObject(retry, getWorld().getWidth()/2, getWorld().getHeight()/2+45);
                    home = new Button("Back to Menu", 1);
                    getWorld().addObject(home, getWorld().getWidth()/2, getWorld().getHeight()/2+135);
                } else if (Settings.Language.equals("de")) {
                    retry = new Button("Erneut versuchen", 1);
                    getWorld().addObject(retry, getWorld().getWidth()/2, getWorld().getHeight()/2+45);
                    home = new Button("Zurück zum Hauptmenü", 1);
                    getWorld().addObject(home, getWorld().getWidth()/2, getWorld().getHeight()/2+135);
                }
            } else if (health == 0 && getWorld().getObjects(Spaceship.class).size() > 1 ){
                getWorld().removeObject(label);
                getWorld().removeObject(exhaust);
                getWorld().removeObject(counter);
                getWorld().removeObject(BoostBar);
                if (Settings.TwoPlayerMode && Settings.ShowPlayerNames) {
                    getWorld().removeObject(pName);
                }
                getWorld().removeObject(this);
                return;
            }
        }
        
        // If boost empty start recharge
        if ((int)bTimer == 0) {
            recharge = true;
        }
        
        // If no longer boosting start recharging if needed
        if(!boost && bTimer < Settings.BoostTime) {
            //                maxBoostTime/rechargeTime
            bTimer = bTimer + (Settings.BoostTime/(double)Settings.BoostRechargeTime);
            // If full stop recharge
            if (bTimer == Settings.BoostTime) {
                recharge = false;
            }
        }
        
        // Set exhaust to the correct location
        exhaust.setLocation(getX(), getY());
        if (boost) {
            exhaust.move(-50);
        } else {
            exhaust.move(-40);
        }
        exhaust.setRotation(getRotation());

        // Place HealthBar under the Spaceship
        counter.setLocation(getX(), getY());
        counter.moveSide(55);
        counter.setRotation(0);
        
        // Place PlayerName on the ship
        if (Settings.TwoPlayerMode && Settings.ShowPlayerNames) {
            pName.setLocation(getX(), getY());
            pName.setRotation(getRotation()+90);
        }
        
        // Place BoostBar under the Spaceship and the HealthBar
        BoostBar.setLocation(getX(), getY());
        BoostBar.moveSide(70);
        BoostBar.setRotation(0);
        BoostBar.setValue((int)bTimer);
        
        // Update Score if not in TitleScreen
        if (!TitleScreen.class.isInstance(getWorld()) ) {
            if (Settings.Language.equals("en")) {
                label.setValue(String.format("Score: %s", Points));
            } else if (Settings.Language.equals("de")) {
                label.setValue(String.format("Punkte: %s", Points));
            }
        }
        
        // Only show for one play
        if(playerNumber == 1 || (playerNumber == 2 && getWorld().getObjects(Spaceship.class).size() == 1)) {
            if (imageTimer > 0) {
                imageTimer--;
            }
            
            // Remove level unlock after timout
            if (imageTimer == 0 && !getWorld().getObjects(Image.class).isEmpty()) {
                getWorld().removeObject(nl);
            }
            // If reached more than 30 points unlock next level
            if (Points >= Settings.LevelUnlockThreshold && !shownMessage && !TitleScreen.class.isInstance(getWorld())) {
                Level level = (Level) getWorld();
                if  (!level.getName().equals("Boss")) {
                    boolean show = false;
                    
                    if (level.getName().equals("Asteroids") && !Settings.FollowersUnlocked && !Settings.UnlockAll) {
                        Settings.FollowersUnlocked = true;
                        show = true;
                    } else if (level.getName().equals("Followers") && !Settings.BattleForEarth && !Settings.UnlockAll) {
                        Settings.BattleOfTwilightGap = true;
                        show = true;
                    } else if (level.getName().equals("Battle for Earth") && !Settings.BattleOfTwilightGap && !Settings.UnlockAll) {
                        Settings.BattleOfTwilightGap = true;
                        show = true;
                    } else if (level.getName().equals("Battle of Twilight Gap") && !Settings.BattleOfTheCity && !Settings.UnlockAll) {
                        Settings.BattleOfTheCity = true;
                        show = true;
                    } else if (level.getName().equals("Battle of the City") && !Settings.HelmsDeep && !Settings.UnlockAll) {
                        Settings.HelmsDeep = true;
                        show = true;
                    } else if (level.getName().equals("Helm's Deep") && !Settings.BossUnlocked && !Settings.UnlockAll) {
                        Settings.BossUnlocked = true;
                        show = true;
                    }
                    shownMessage = true;
                    if (show) {
                        nl = new Image();
                        nl.setImage("newLevel.png");
                        getWorld().addObject(nl, 200, getWorld().getHeight()-50);
                        imageTimer = 1000;
                    }
                }
            }
        }
    }
    /**
     * Check for key presses and handle them accordingly
     */
    public void checkKeyPress() {
        //System.out.println(1-((1/(double)accelerationTime)*(double)acceleration));
        //System.out.println(1.5-((1.5/(double)Settings.AccelerationTime)*(double)acceleration));
        
        // If "W" is pressed or it is still decelerating and "shift" not pressed
        if ((Greenfoot.isKeyDown(forwardKey) || acceleration < Settings.AccelerationTime) && !Greenfoot.isKeyDown(boostKey)) {
            defaultMove(true);
        }
        // If moving backwards is enabled move back on "s"
        if (Greenfoot.isKeyDown(backwardKey) && Settings.CanMoveBackward) {move(-Settings.NormalSpeed);}
        // Turn left
        if (Greenfoot.isKeyDown(leftKey)) {turn(-1);}
        // Turn right
        if (Greenfoot.isKeyDown(rightKey)) {turn(1);}
        if (timer > 0) {timer--;}
        // If "space" pressed and the shoot timer is on 0 and the Spaceship can shoot, shoot
        if(Greenfoot.isKeyDown(shootKey) && timer == 0 && Settings.CanShoot) {
            // Spawn with Spaceship rotation and in front of the Spaceship
            Shots actor = new Shots(getRotation(), false, 1);
            getWorld().addObject(actor, getX(), getY());
            actor.move(50);
            actor.setColor(soColor);
            timer = Settings.ShotsDelay;
        }
        // If "shift" pressed,the Spaceship can boost, the boost isn't empty and the boost isn't currently recharging
        if(Greenfoot.isKeyDown(boostKey) && Settings.CanBoost && bTimer > 0 && !recharge) {
            exhaust.setEngine(true); 
            boost = true;
            
            // If still accelerating (is over 0) move according to current speed, otherwise move maxspeed
            if(acceleration > 0){
                // maxSpeed - ((maxSpeed/AccelerationTime)*acceleration)
                move(Settings.BoostSpeed-((Settings.BoostSpeed/(double)Settings.AccelerationTime)*(double)acceleration));
            }else{
                move(Settings.BoostSpeed);
            }
            // Pressing "Shift", further accelerate
            if(acceleration > 0 && Greenfoot.isKeyDown(boostKey)){
                acceleration--;
            }
            // If neither "W" nor "shift", decelerate
            if(acceleration < Settings.AccelerationTime && !Greenfoot.isKeyDown(boostKey) && !Greenfoot.isKeyDown(forwardKey)){
                exhaust.setEngine(false);
                acceleration++;
            }
            
            bTimer--;
            exhaust.setBoost(true);
            
        } else if (Greenfoot.isKeyDown(boostKey) && Settings.CanBoost && recharge) { 
            // If "shift", can boost and it is currently recharging decelerate or move if BoostCanDrive
            exhaust.setBoost(false);
            boost = false;
            defaultMove(Settings.BoostCanDrive);
        }else {
            exhaust.setBoost(false);
            boost = false;
        }
    }
    /**
     * Moving and acceleration method
     */
    public void defaultMove(boolean move){
        exhaust.setEngine(true); 
        if(!boost){
            // If still accelerating (is over 0) move according to current speed, otherwise move maxspeed
            if(acceleration > 0){
                // maxSpeed - ((maxSpeed/AccelerationTime)*acceleration)
                move(Settings.NormalSpeed-((Settings.NormalSpeed/(double)Settings.AccelerationTime)*(double)acceleration));
            }else{
                move(Settings.NormalSpeed);
            }
            // Pressing ("W" or "Shift") and move is true, further accelerate
            if(acceleration > 0 && (Greenfoot.isKeyDown(forwardKey) || Greenfoot.isKeyDown(boostKey)) && move){
                acceleration--;
            }
        }
        // if neither "W" nor ("shift" and not (boosting or shift can also drive normally)), decelerate
        if(acceleration < Settings.AccelerationTime && !Greenfoot.isKeyDown(forwardKey) && !(Greenfoot.isKeyDown(boostKey) && (boost || Settings.BoostCanDrive))){
            exhaust.setEngine(false);
            acceleration++;
        }
    }
    /**
     * This method is called by the Greenfoot system when this actor has been inserted into the world.
     */
    protected void addedToWorld​(World world) {
        // Set selected color of the Spaceship
        setImage(String.format("spaceship(%s).png", scColor.toLowerCase()));
        
        // Prepare Exhaust
        exhaust = new Exhaust();
        getWorld().addObject(exhaust, getWorld().getWidth()/2, getWorld().getHeight()/2);
        exhaust.setSpeed(30);
        exhaust.setColor(ecColor);
        exhaust.setStartSpeed(3);
        
        exhaust.setRotation(-90);
        exhaust.setLocation(getX(), getY());
        exhaust.move(-40);
        
        // Set worldspeed, turn upwards and prepare variables
        Greenfoot.setSpeed(60);
        turn(-90);
        bTimer = Settings.BoostTime;
        acceleration = Settings.AccelerationTime;
        
        // Show the Score in the top right corner if not in TitleScreen
        if (!TitleScreen.class.isInstance(getWorld()) ) {
            if (Settings.Language.equals("en")) {
                label = new Label("Score: 0", 50);
                getWorld().addObject(label, getWorld().getWidth()-110, 40);
            } else if (Settings.Language.equals("de")) {
                label = new Label("Punkte: 0", 50);
                getWorld().addObject(label, getWorld().getWidth()-110, 40);
            }
        }
        
        //Show playerName if more than one player
        if (Settings.TwoPlayerMode && Settings.ShowPlayerNames) {
            pName = new Label(playerName, 25);
            getWorld().addObject(pName, getX(), getY());
            pName.setRotation(getRotation()+90);
        }
        
        // Prepare the HealthBar
        counter = new Bar("", "", 100, 100);
        getWorld().addObject(counter, getWorld().getWidth()/2, getWorld().getHeight()/2);
        counter.setRotation(90);
        counter.moveSide(40);
        
        // Prepare the BoostBar
        BoostBar = new Bar("", "", Settings.BoostTime, Settings.BoostTime);
        BoostBar.setBreakPercent(15);
        BoostBar.setSafeColor(Color.YELLOW);
        getWorld().addObject(BoostBar, getWorld().getWidth()/2, getWorld().getHeight()/2);
        
    }
}


