import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * WorldHelper
 * Provides helpfull method for the different worlds
 * 
 * @author Jannis Martensen
 * @version 1.0
 */
public class WorldHelper extends World
{
    /**
     * Constructor for objects of class WorldHelper.
     */
    public WorldHelper()
    {    
        // Makes all worlds this size
        super(1750, 900, 1);
    }
    /**
     * Method to spawn a maximum given amount of Asteroids
     * Bigger Asteroids count as multiple smaller ones
     */
    public void Asteroids(int Counter, int maxAsteroids){
        Counter = 0;
        for (Object i :  getObjects(Asteroid.class)) {
            Asteroid ast = (Asteroid) i;
            switch (ast.getType()) {
                case 1:
                    Counter++;
                case 2:
                    Counter += 2;
                case 3:
                    Counter += 3;
                default:
                    Counter++;
            }
        }
        // Only spawns a new Asteroid if there is less than given amount
        if (HelperClass.chance(0.75) &&  Counter < maxAsteroids) {
            int type = Greenfoot.getRandomNumber(100);
            
            if (type < 60) { // 60% chance for a small one
                addObject(new Asteroid(1), 1, 1);
            } else if (90 > type && type >= 60) { // 30% chance for a medium one
                addObject(new Asteroid(2), 1, 1);
            } else { // 10% chance for a big one
                addObject(new Asteroid(3), 1, 1);
            }
        }
    }
    /**
     * Method to spawn a maximum given amount of EasyEnemies
     */
    public void EasyEnemies(int maxEasyEnemies){
        if (getObjects(EasyEnemies.class).size() < maxEasyEnemies) {
            addObject(new EasyEnemies(), 1, 1);
        }
    }
    /**
     * Method to spawn a maximum given amount of AttackerEnemies
     */
    public void AttackerEnemies(int maxAttackerEnemies){
        if (getObjects(AttackerEnemies.class).size() < maxAttackerEnemies) {
            addObject(new AttackerEnemies(), 1,1);
        }
    }
    /**
     * Method to spawn a maximum given amount of BossEnemies
     */
    public void BossEnemies(int maxBossEnemies){
        if (getObjects(BossEnemies.class).size() < maxBossEnemies) {
            addObject(new BossEnemies(false), 1,1);
        }
    }
    /**
     * Increase enemy speed linearly with score up to a max
     */
    public void IncreaseEnemySpeed(Label SpeedLabel) {
        int points      =    Spaceship.Points;
        int maxPoints   =    Settings.ScaleSpeedMaxPoints;
        
        // Calculate only if enabled and the points are not over the threshold
        if (Settings.ScaleEnemiesSpeed && points <= maxPoints) {
            
            // Get all important values
            double EEMin    =    Settings.EasyEnemiesMinSpeed;
            double EEMax    =    Settings.EasyEnemiesMaxSpeed;
            double AEMin    =    Settings.AttackerEnemiesMinSpeed;
            double AEMax    =    Settings.AttackerEnemiesMaxSpeed;
            double BEMin    =    Settings.BossEnemiesMinSpeed;
            double BEMax    =    Settings.BossEnemiesMaxSpeed;
            
            // Set speed                           minPoints, (double) maxPoints, EEMin, EEMax, (double) points
            Settings.EasyEnemiesSpeed       =   mapRange(0.0, (double) maxPoints, EEMin, EEMax, (double) points);
            Settings.AttackerEnemiesSpeed   =   mapRange(0.0, (double) maxPoints, AEMin, AEMax, (double) points);
            Settings.BossEnemiesSpeed       =   mapRange(0.0, (double) maxPoints, BEMin, BEMax, (double) points);
            
            // Get percentage and display at bottom of screen
            double SpeedPercentage          =   mapRange(0.0, (double) maxPoints,  0.0,  100.0, (double) points);
            if (Settings.Language.equals("en")) {
                SpeedLabel.setValue(String.format("Speed: %s%%", (int)SpeedPercentage));
            } else if (Settings.Language.equals("de")) {
                SpeedLabel.setValue(String.format("Tempo: %s%%", (int)SpeedPercentage));
            }
        }
    }
    /**
     * Maps one number range to another 
     */
    public static double mapRange(double inStart, double inEnd, double outStart, double outEnd, double value){
        return outStart + ((value - inStart)*(outEnd - outStart))/(inEnd - inStart);
    }
    /**
     * Set the background color of a World
     */
    public void setWorldColor(int c1, int c2, int c3, int c4) {
        GreenfootImage image = new GreenfootImage(getWidth(), getHeight());
        image.setColor(new Color(c1, c2, c3, c4));
        image.fill();
        setBackground(image);
    }
}
